--[[
SimpleMD - Copyright (C) 2007 Michael Joseph Murray aka Starinnia of Lothar
contact: codemaster2010@gmail.com

This addon started as a simple text-warning system for use in my guild.
I released it to wowace and Curse in the hopes it would help other hunters.

Cooldown bars were recently requested and implemented.
Rather than re-invent the wheel, I looked at other addons for a useful bar implementation,
MendWatch by Polima was the source of the bars used in this addon.

-----------------------------------------------------------------------------------------
This addon is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.0 of the License, or (at your option) any later version.

This addon is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
]]

-- if player class is not hunter do not load anything
if (select(2, UnitClass("player"))) ~= "HUNTER" then return end

local L = AceLibrary("AceLocale-2.2"):new("SimpleMD")
local PlayerName = UnitName("player").." "
local rev = tonumber(("$Revision: 56407 $"):match("%d+"))
local addonID = "SimpleMD - r"..rev

SimpleMD = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "Sink-1.0")
local DD = AceLibrary("Dewdrop-2.0")
local SML = AceLibrary("SharedMedia-1.0")

function SimpleMD:OnInitialize()	
	local defaults = {
		messageGain = L["<player> cast Misdirection on <target>"],
		messageFade = L["Misdirection fades from <target>"],
		r = 1.0,
		g = 1.0,
		b = 1.0,
		channel = "",
		showInChannel = false,
		showInChat = false,
		showOnScreen = true,
		showFloating = false,
		showSelf = true,
		showRW = false,
		showGroup = false,
		showBars = true,
		showGain = true,
		showFade = true,
		ready = true,
		ten = true,
		thirty = true,
		sixty = false,
		texture = "Blizzard",
		font = "Friz Quadrata TT",
		barR = 1.0,
		barG = 1.0,
		barB = 1.0,
		BGr = 1.0,
		BGg = 1.0,
		BGb = 1.0,
		txtR = 1.0,
		txtG = 1.0,
		txtB = 1.0,
		bg_alpha = 1.0,
		textSize = 11.08,
		barScale = 1.0,
		barWidth = 195.00,
		barHeight = 17.00,
		growUp = false,
		reverse = false,
		stay = false,
		position = {},
	}
	
	SimpleMD:RegisterDB("SimplemdDB")
	SimpleMD:RegisterDefaults('profile', defaults)
	SimpleMD:SetCommPrefix("SimpleMD")
	
	
	self.anchor, self.frame = self:CreateAnchor(addonID)
	self.anchor.timers = {}
	self.timercount = 0
	self.recyclePile = {}
	self.idents = {}
	DD:Register(self.anchor, "children", self.MenuRequest, 'cursorX', true, 'cursorY', true)
end

function SimpleMD:OnEnable()
	self.tex = self:GetTexture(L["Misdirection"])
	
	self:RegisterComm("SimpleMD", "GROUP", "OnCommReceive")
	
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD")
	self:RegisterEvent("SIMPLEMD_TESTBARS")
end

function SimpleMD:OnDisable()
	self:UnregisrerAllEvents()
	self.tex = nil
end

function SimpleMD:OnCommReceive(prefix, sender, distribution, message)	
	if self.db.profile.showInChat and message ~= "GAIN" and message ~= "READINESS" then
		self:Print(message)
	end
	if self.db.profile.showOnScreen and message ~= "GAIN" and message ~= "READINESS" then
		UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showFloating and message ~= "GAIN" and message ~= "READINESS" then
		self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
	if message == "GAIN" and self.db.profile.showBars then
		self:CreateTimer(sender.." ", nil, 120, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp, self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, self.db.profile.bg_alpha, self.db.profile.BGr, self.db.profile.BGg, self.db.profile.BGb)
	end
	if message == "READINESS" and self.db.profile.showBars then
		self:ReadinessCheck(sender)
	end
end

function SimpleMD:SendMessage(message, id)	
	SimpleMD:SendCommMessage("GROUP", message)
	
	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChannel and self.db.profile.channel ~= "" then
			SendChatMessage(message, "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
		end
	
		if self.db.profile.showFloating and self.db.profile.showSelf and id ~= 1 then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		if self.db.profile.showGroup and GetNumRaidMembers() == 0 then
			SendChatMessage(message, "PARTY")
		end
		if self.db.profile.showGroup and GetNumRaidMembers() > 5 then
			SendChatMessage(message, "RAID")
		end
	end
end

--check to see if spell is Misdirection, if it is set the spell target
function SimpleMD:UNIT_SPELLCAST_SENT(arg1, arg2, arg3, arg4)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self.SpellTarget = arg4
		end
	end
end

-- if spellcast succeeds call the gain function and schedule cooldown events
-- needed to make sure messages are not sent when target is out of LOS or out of range or invalid
function SimpleMD:UNIT_SPELLCAST_SUCCEEDED(arg1, arg2, arg3)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self:Make_CD_Bar()
			if self.db.profile.showGain then
				self:Gain()
			end
			if self.db.profile.sixty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 60, 60) --60 second warning
			end
			if self.db.profile.thirty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 90, 30) --30 second warning
			end
			if self.db.profile.ten then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 110, 10) -- 10 second warning
			end
			if self.db.profile.ready then
			self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 120, 0) -- cooldown finished
			end
		end
		--account for the readiness talent
		if arg2 == L["Readiness"] then
			self:CancelAllScheduledEvents()
			local temp = PlayerName .. " " .. L["used Readiness! MD ready!"]
			self:SendMessage(temp)
			SimpleMD:SendCommMessage("GROUP", "READINESS")
			if self.db.profile.showSelf and self.db.profile.showBars then
				self:ReadinessCheck(PlayerName)
			end
		end
	end
end

function SimpleMD:CHAT_MSG_SPELL_AURA_GONE_SELF(arg1)
	if arg1 == L["Misdirection fades from you."] then
		if self.db.profile.showFade then
			self:Fade()
		end
	end
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD(arg1)
	if not UnitIsDeadOrGhost("player") then
		if arg1 == 60 then
			local temp = PlayerName .. " " .. L["has 60 seconds left on MD cooldown."]
			self:SendMessage(temp, 2)
		end
		if arg1 == 30 then
			local temp = PlayerName .. " " .. L["has 30 seconds left on MD cooldown."]
			self:SendMessage(temp, 2)
		end
		if arg1 == 10 then
			local temp = PlayerName .. " " .. L["has 10 seconds left on MD cooldown!"]
			self:SendMessage(temp, 2)
		end
		if arg1 == 0 then
			local temp = PlayerName .. " " .. L["has MD ready!"]
			self:SendMessage(temp, 2)
		end
	end
end

function SimpleMD:SIMPLEMD_TESTBARS()
	self:ClearTimers()
end

-- called on successful MD casts
function SimpleMD:Gain()
	local message = string.gsub(self.db.profile.messageGain, "<player>", PlayerName)
	message = string.gsub(message, "<target>", self.SpellTarget)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		if (IsRaidLeader() or IsRaidOfficer()) and self.db.profile.showRW then
			SendChatMessage(message, "RAID_WARNING")
		end
		
		self:SendMessage(message, 1)
	end
end

function SimpleMD:Make_CD_Bar()
	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		self:SendCommMessage("GROUP", "GAIN")
		if self.db.profile.showSelf and self.db.profile.showBars then
			self:CreateTimer(PlayerName, nil, 120, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp, self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, self.db.profile.bg_alpha, self.db.profile.BGr, self.db.profile.BGg, self.db.profile.BGb)
		end
	end
end

-- called on Misdirection fading from self
function SimpleMD:Fade()
	local message = string.gsub(self.db.profile.messageFade, "<target>", self.SpellTarget)
	message = string.gsub(message, "<player>", PlayerName)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		
		self:SendMessage(message, 1)
	end
end

-- test broadcast for making sure colors or channel is correct
function SimpleMD:Test()
	if self.db.profile.showInChat then
		self:Print(L["Testing Chat Frame Printing"])
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(L["Testing On Screen Printing"], self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showInChannel and channel ~= "" then
		SendChatMessage(L["Testing Channel Printing"], "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
	end
	if self.db.profile.showFloating then
		self:Pour(L["Testing Floating Message"], self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
end

function SimpleMD:ToggleAnchor()
	if self.anchor:IsVisible() then
		self.anchor:Hide()
	else
		self.anchor:Show()
	end
end

function SimpleMD:DisableBars()
	if self.db.profile.showBars then
		self.db.profile.showBars = false
	else
		self.db.profile.showBars = true
	end
	
	SimpleMD:ClearTimers()
end

function SimpleMD:GetTexture(name)
	local i = 1
	while true do
		local spell = GetSpellName(i, BOOKTYPE_SPELL)
		if not spell then break
		elseif spell == name then
			tex = GetSpellTexture(i, BOOKTYPE_SPELL)
			return tex
		end
		i = i + 1
	end
end

function SimpleMD:BarTest()
	self:ClearTimers()
	self:CreateTimer("Test1 ", "temp", 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp, self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, self.db.profile.bg_alpha, self.db.profile.BGr, self.db.profile.BGg, self.db.profile.BGb)
	self:CreateTimer("test2 ", nil, 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp, self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, self.db.profile.bg_alpha, self.db.profile.BGr, self.db.profile.BGg, self.db.profile.BGb)
	self:ScheduleEvent("SIMPLEMD_TESTBARS", 30)
end
